<?php

/**
 * @file
 * Filter hook implementations and callbacks.
 */

/**
 * Non-HTML markup language filters that generate HTML.
 */
define('TYPE_MARKUP_LANGUAGE', 0);

/**
 * HTML tag and attribute restricting filters to prevent XSS attacks.
 */
define('TYPE_HTML_RESTRICTOR', 1);

/**
 * Reversible transformation filters.
 */
define('TYPE_TRANSFORM_REVERSIBLE', 2);

/**
 * Irreversible transformation filters.
 */
define('TYPE_TRANSFORM_IRREVERSIBLE', 3);

/**
 * Implements hook_filter_format_insert().
 */
function editor_filter_format_insert($format) {
  _editor_format_save($format);
}

/**
 * Implements hook_filter_format_update().
 */
function editor_filter_format_update($format) {
  _editor_format_save($format);
}

/**
 * Helper function to save editor information on text format insert/update.
 *
 * @param object $format
 *   An object representing the text format.
 */
function _editor_format_save($format) {
  if (!isset($format->editor)) {
    $format->editor = '';
  }

  if (!isset($format->editor_settings)) {
    $format->editor_settings = array();
  }

  if (!isset($format->image_upload)) {
    $format->image_upload = array();
  }

  // Add 'support' for features.module which will store editor and image upload
  // settings as a serialized array since it gets its information from
  // filter_format_load()  and filter_formats() which are hard-coded to ensure
  // that the standard filter settings are serialized but cannot be altered to
  // do the same for editor settings.
  // @todo Remove this when we can ensure that settings are unserialized.

  // Unserialize the editor settings when necessary. This prevents the editor
  // settings from being serialized twice.
  if (is_string($format->editor_settings)) {
    $editor_settings = $format->editor_settings;

    if ($editor_settings == serialize(FALSE) || @unserialize($editor_settings) !== FALSE) {
      $format->editor_settings = unserialize($editor_settings);
    }
  }

  // Unserialize the image upload settings when necessary. This prevents the
  // image upload settings from being serialized twice.
  if (is_string($format->image_upload)) {
    $image_upload_settings = $format->image_upload;

    if ($image_upload_settings == serialize(FALSE) || @unserialize($image_upload_settings) !== FALSE) {
      $format->image_upload = unserialize($image_upload_settings);
    }
  }

  // Insert or update the text format.
  db_merge('filter_format')
    ->key(array('format' => $format->format))
    ->fields(array(
      'editor' => $format->editor,
      'editor_settings' => serialize($format->editor_settings),
      'image_upload' => serialize($format->image_upload),
    ))
    ->execute();
}

/**
 * Implements hook_filter_info().
 */
function editor_filter_info() {
  $filters['editor_caption'] = array(
    'title' => t('Caption images'),
    'description' => t('Uses a <code>data-caption</code> attribute on <code>&lt;img&gt;</code> tags to caption images.'),
    'type' => TYPE_TRANSFORM_REVERSIBLE,
    'process callback' => '_editor_caption',
    'tips callback' => '_editor_caption_tips',
    'weight' => 4,
  );
  $filters['editor_align'] = array(
    'title' => t('Align images'),
    'description' => t('Uses a <code>data-align</code> attribute on <code>&lt;img&gt;</code> tags to align images.'),
    'type' => TYPE_TRANSFORM_REVERSIBLE,
    'process callback' => '_editor_align',
    'tips callback' => '_editor_align_tips',
    'weight' => 5,
  );

  return $filters;
}

/**
 * Implements callback_filter_process().
 *
 * Replace img data-caption attributes with figure and figcaption elements.
 */
function _editor_caption($text) {
  $result = $text;

  if (stristr($text, 'data-caption') !== FALSE) {
    // Load the text as a DOM object for manipulation.
    $dom = filter_dom_load($text);
    $xpath = new DOMXPath($dom);
    $html_filter = new stdClass();
    $html_filter->settings = array(
      'allowed_html' => '<a href hreflang target rel> <em> <strong> <cite> <code> <br>',
      'filter_html_help' => FALSE,
      'filter_html_nofollow' => FALSE,
    );
    foreach ($xpath->query('//*[@data-caption]') as $node) {
      // Read the data-caption attribute's value, then delete it.
      $caption = $node->getAttribute('data-caption');
      $node->removeAttribute('data-caption');

      // Sanitize caption: decode HTML encoding, limit allowed HTML tags; only
      // allow inline tags that are allowed by default, plus <br>.
      $caption = decode_entities($caption);
      $raw_caption = $caption;
      $caption = _editor_filter_html($caption, $html_filter);

      // The caption must be non-empty - however the Media Embed CKEditor
      // plugin uses a single space to represent a newly added caption. The
      // HTML filter will transform this into an empty string and prevent the
      // content editor from adding a new caption. To allow for this we treat
      // a raw caption value of ' ' as valid and adding the wrapping figure
      // element.
      // @see core/modules/media/js/plugins/drupalmedia/plugin.js
      if (mb_strlen($caption) === 0 && $raw_caption !== ' ') {
        continue;
      }

      // Given the updated node and caption: re-render it with a caption, but
      // bubble up the value of the class attribute of the captioned element,
      // this allows it to collaborate with e.g. the filter_align filter.
      $tag = $node->tagName;
      $classes = $node->getAttribute('class');
      $node->removeAttribute('class');
      $node = ($node->parentNode->tagName === 'a') ? $node->parentNode : $node;
      $filter_caption = array(
        '#theme' => 'editor_caption',
        // We pass the unsanitized string because this is a text format
        // filter, and after filtering, we always assume the output is safe.
        // @see \Drupal\filter\Element\ProcessedText::preRenderText()
        '#node' => $node->C14N(),
        '#tag' => $tag,
        '#caption' => $caption,
        '#classes' => $classes,
      );
      $altered_html = drupal_render($filter_caption);

      // Load the altered HTML into a new DOMDocument and retrieve the element.
      $updated_nodes = filter_dom_load($altered_html)->getElementsByTagName('body')
        ->item(0)
        ->childNodes;

      foreach ($updated_nodes as $updated_node) {
        // Import the updated node from the new DOMDocument into the original
        // one, importing also the child nodes of the updated node.
        $updated_node = $dom->importNode($updated_node, TRUE);
        $node->parentNode->insertBefore($updated_node, $node);
      }
      // Finally, remove the original data-caption node.
      $node->parentNode->removeChild($node);
    }

    $result = filter_dom_serialize($dom);
  }

  return $result;
}

/**
 * Implements callback_filter_tips().
 *
 * Provides help for the caption filter.
 */
function _editor_caption_tips($filter, $format, $long = FALSE) {
  if ($long) {
    return t('
        <p>You can caption images, videos, blockquotes, and so on. Examples:</p>
        <ul>
            <li><code>&lt;img src="" data-caption="This is a caption" /&gt;</code></li>
            <li><code>&lt;video src="" data-caption="The Drupal Dance" /&gt;</code></li>
            <li><code>&lt;blockquote data-caption="Dries Buytaert"&gt;Drupal is awesome!&lt;/blockquote&gt;</code></li>
            <li><code>&lt;code data-caption="Hello world in JavaScript."&gt;alert("Hello world!");&lt;/code&gt;</code></li>
        </ul>');
  }
  else {
    return t('You can caption images (<code>data-caption="Text"</code>), but also videos, blockquotes, and so on.');
  }
}

/**
 * Implements callback_filter_process().
 *
 * Uses a data-align attribute on <img> tags to align images.
 */
function _editor_align($text) {
  $result = $text;

  if (stristr($text, 'data-align') !== FALSE) {
    $dom = filter_dom_load($text);
    $xpath = new DOMXPath($dom);
    foreach ($xpath->query('//*[@data-align]') as $node) {
      // Read the data-align attribute's value, then delete it.
      $align = $node->getAttribute('data-align');
      $node->removeAttribute('data-align');

      // If one of the allowed alignments, add the corresponding class.
      if (in_array($align, array('left', 'center', 'right'))) {
        $classes = $node->getAttribute('class');
        $classes = (strlen($classes) > 0) ? explode(' ', $classes) : array();
        $classes[] = 'align-' . $align;
        $node->setAttribute('class', implode(' ', $classes));
      }
    }

    $result = filter_dom_serialize($dom);
  }

  return $result;
}

/**
 * Implements callback_filter_tips().
 *
 * Provides help for the align filter.
 */
function _editor_align_tips($filter, $format, $long = FALSE) {
  if ($long) {
    return t('
        <p>You can align images, videos, blockquotes and so on to the left, right or center. Examples:</p>
        <ul>
          <li>Align an image to the left: <code>&lt;img src="" data-align="left" /&gt;</code></li>
          <li>Align an image to the center: <code>&lt;img src="" data-align="center" /&gt;</code></li>
          <li>Align an image to the right: <code>&lt;img src="" data-align="right" /&gt;</code></li>
          <li>… and you can apply this to other elements as well: <code>&lt;video src="" data-align="center" /&gt;</code></li>
        </ul>');
  }
  else {
    return t('You can align images (<code>data-align="center"</code>), but also videos, blockquotes, and so on.');
  }
}

/**
 * Implements hook_filter_info_alter().
 */
function editor_filter_info_alter(&$info) {
  // Add an allowed HTML callback to the core 'filter_html' filter which enables
  // it to specify a whitelist of approved HTML tags/attributes.
  if (isset($info['filter_html'])) {
    $info['filter_html']['process callback'] = '_editor_filter_html';
    $info['filter_html']['default settings']['allowed_html'] = '<a href hreflang> <em> <strong> <cite> <blockquote cite> <code> <ul type> <ol start type> <li> <dl> <dt> <dd> <h2 id> <h3 id> <h4 id> <h5 id> <h6 id> <p class="text-align-left text-align-center text-align-right text-align-justify"> <img src alt data-entity-type data-entity-id data-align data-caption>';
    $info['filter_html']['allowed html callback'] = 'editor_filter_html_get_html_restrictions';
    $info['filter_html']['settings callback'] = '_editor_html_settings';
  }

  // Non-HTML markup language filters that generate HTML.
  if (isset($info['filter_autop'])) {
    $info['filter_autop']['type'] = TYPE_MARKUP_LANGUAGE;
  }
  if (isset($info['filter_url'])) {
    $info['filter_url']['type'] = TYPE_MARKUP_LANGUAGE;
  }

  // Irreversible transformation filters.
  if (isset($info['filter_htmlcorrector'])) {
    $info['filter_htmlcorrector']['type'] = TYPE_TRANSFORM_IRREVERSIBLE;
  }

  // HTML tag and attribute restricting filters to prevent XSS attacks.
  if (isset($info['filter_html'])) {
    $info['filter_html']['type'] = TYPE_HTML_RESTRICTOR;
  }
  if (isset($info['filter_html_escape'])) {
    $info['filter_html_escape']['type'] = TYPE_HTML_RESTRICTOR;
    $info['filter_html_escape']['allowed html callback'] = '_filter_html_escape_allowed_html';
  }

  // Add a default 'allowed html callback' to filters that don't provide one.
  foreach ($info as $filter_name => $settings) {
    if (!isset($settings['allowed html callback'])) {
      $info[$filter_name]['allowed html callback'] = '_filter_default_allowed_html';
    }
  }
}

/**
 * Allowed HTML callback for the 'filter_html_escape' filter.
 *
 * @param $filter
 *    An object representing the filter.
 */
function _filter_html_escape_allowed_html($filter) {
  // Nothing is allowed.
  return array('allowed' => array());
}

/**
 * Default allowed HTML callback for filters that don't provide one.
 *
 * @param $filter
 *    An object representing the filter.
 */
function _filter_default_allowed_html($filter) {
  return FALSE;
}

/**
 * Implements callback_filter_process().
 *
 * Provides filtering of input into accepted HTML.
 */
function _editor_filter_html($text, $filter) {
  $restrictions = editor_filter_html_get_html_restrictions($filter);
  // Split the work into two parts. For filtering HTML tags out of the content
  // we rely on the well-tested Xss::filter() code. Since there is no '*' tag
  // that needs to be removed from the list.
  unset($restrictions['allowed']['*']);
  $text = Xss::filter($text, array_keys($restrictions['allowed']));
  // After we've done tag filtering, we do attribute and attribute value
  // filtering as the second part.
  return trim(editor_filter_html_filter_attributes($text, $filter));
}

/**
 * Provides filtering of tag attributes into accepted HTML.
 *
 * @param string $text
 *   The HTML text string to be filtered.
 *
 * @return string
 *   Filtered HTML with attributes filtered according to the settings.
 */
function editor_filter_html_filter_attributes($text, $filter) {
  $restrictions = editor_filter_html_get_html_restrictions($filter);
  $global_allowed_attributes = array_filter($restrictions['allowed']['*']);
  unset($restrictions['allowed']['*']);

  // Apply attribute restrictions to tags.
  $html_dom = filter_dom_load($text);
  $xpath = new DOMXPath($html_dom);
  foreach ($restrictions['allowed'] as $allowed_tag => $tag_attributes) {
    // By default, no attributes are allowed for a tag, but due to the
    // globally allowed attributes, it is impossible for a tag to actually
    // completely disallow attributes.
    if ($tag_attributes === FALSE) {
      $tag_attributes = array();
    }
    $allowed_attributes = array('exact' => array(), 'prefix' => array());
    foreach (($global_allowed_attributes + $tag_attributes) as $name => $values) {
      // A trailing * indicates wildcard, but it must have some prefix.
      if (substr($name, -1) === '*' && $name[0] !== '*') {
        $allowed_attributes['prefix'][str_replace('*', '', $name)] = editor_filter_html_prepare_attribute_values($values);
      }
      else {
        $allowed_attributes['exact'][$name] = editor_filter_html_prepare_attribute_values($values);
      }
    }
    krsort($allowed_attributes['prefix']);

    // Find all matching elements that have any attributes and filter the
    // attributes by name and value.
    foreach ($xpath->query('//' . $allowed_tag . '[@*]') as $element) {
      editor_filter_html_filter_element_attributes($element, $allowed_attributes);
    }
  }

  if ($filter->settings['filter_html_nofollow']) {
    $links = $html_dom->getElementsByTagName('a');
    foreach ($links as $link) {
      $link->setAttribute('rel', 'nofollow');
    }
  }
  $text = filter_dom_serialize($html_dom);

  return trim($text);
}

/**
 * Filters attributes on an element according to a list of allowed values.
 *
 * @param \DOMElement $element
 *   The element to be processed.
 * @param array $allowed_attributes
 *   The list of allowed attributes as an array of names and values.
 */
function editor_filter_html_filter_element_attributes(\DOMElement $element, array $allowed_attributes) {
  $modified_attributes = array();
  foreach ($element->attributes as $name => $attribute) {
    // Remove attributes not in the list of allowed attributes.
    $allowed_value = editor_filter_html_find_allowed_value($allowed_attributes, $name);
    if (empty($allowed_value)) {
      $modified_attributes[$name] = FALSE;
    }
    elseif ($allowed_value !== TRUE) {
      // Check the list of allowed attribute values.
      $attribute_values = preg_split('/\s+/', $attribute->value, -1, PREG_SPLIT_NO_EMPTY);
      $modified_attributes[$name] = array();
      foreach ($attribute_values as $value) {
        if (editor_filter_html_find_allowed_value($allowed_value, $value)) {
          $modified_attributes[$name][] = $value;
        }
      }
    }
  }
  // If the $allowed_value was TRUE for an attribute name, it does not
  // appear in this array so the value on the DOM element is left unchanged.
  foreach ($modified_attributes as $name => $values) {
    if ($values) {
      $element->setAttribute($name, implode(' ', $values));
    }
    else {
      $element->removeAttribute($name);
    }
  }
}

/**
 * Helper function to handle prefix matching.
 *
 * @param array $allowed
 *   Array of allowed names and prefixes.
 * @param string $name
 *   The name to find or match against a prefix.
 *
 * @return bool|array
 */
function editor_filter_html_find_allowed_value(array $allowed, $name) {
  if (isset($allowed['exact'][$name])) {
    return $allowed['exact'][$name];
  }
  // Handle prefix (wildcard) matches.
  foreach ($allowed['prefix'] as $prefix => $value) {
    if (strpos($name, $prefix) === 0) {
      return $value;
    }
  }
  return FALSE;
}

/**
 * Helper function to prepare attribute values including wildcards.
 *
 * Splits the values into two lists, one for values that must match exactly
 * and the other for values that are wildcard prefixes.
 *
 * @param bool|array $attribute_values
 *   TRUE, FALSE, or an array of allowed values.
 *
 * @return bool|array
 */
function editor_filter_html_prepare_attribute_values($attribute_values) {
  if ($attribute_values === TRUE || $attribute_values === FALSE) {
    return $attribute_values;
  }
  $result = array('exact' => array(), 'prefix' => array());
  foreach ($attribute_values as $name => $allowed) {
    // A trailing * indicates wildcard, but it must have some prefix.
    if (substr($name, -1) === '*' && $name[0] !== '*') {
      $result['prefix'][str_replace('*', '', $name)] = $allowed;
    }
    else {
      $result['exact'][$name] = $allowed;
    }
  }
  krsort($result['prefix']);
  return $result;
}

/**
 * Returns HTML allowed by this filter's configuration.
 *
 * May be implemented by filters of the TYPE_HTML_RESTRICTOR
 * type, this won't be used for filters of other types; they should just
 * return FALSE.
 *
 * This callback function is only necessary for filters that strip away HTML
 * tags (and possibly attributes) and allows other modules to gain insight in
 * a generic manner into which HTML tags and attributes are allowed by a
 * format.
 *
 * @return array|false
 *   A nested array with *either* of the following keys:
 *     - 'allowed': (optional) the allowed tags as keys, and for each of those
 *       tags (keys) either of the following values:
 *       - TRUE to indicate any attribute is allowed
 *       - FALSE to indicate no attributes are allowed
 *       - an array to convey attribute restrictions: the keys must be
 *         attribute names (which may use a wildcard, e.g. "data-*"), the
 *         possible values are similar to the above:
 *           - TRUE to indicate any attribute value is allowed
 *           - FALSE to indicate the attribute is forbidden
 *           - an array to convey attribute value restrictions: the key must
 *             be attribute values (which may use a wildcard, e.g. "xsd:*"),
 *             the possible values are TRUE or FALSE: to mark the attribute
 *             value as allowed or forbidden, respectively
 *     -  'forbidden_tags': (optional) the forbidden tags
 *
 *   There is one special case: the "wildcard tag", "*": any attribute
 *   restrictions on that pseudotag apply to all tags.
 *
 *   If no restrictions apply, then FALSE must be returned.
 *
 *   Here is a concrete example, for a very granular filter:
 *     @code
 *     array(
 *       'allowed' => array(
 *         // Allows any attribute with any value on the <div> tag.
 *         'div' => TRUE,
 *         // Allows no attributes on the <p> tag.
 *         'p' => FALSE,
 *         // Allows the following attributes on the <a> tag:
 *         //  - 'href', with any value;
 *         //  - 'rel', with the value 'nofollow' value.
 *         'a' => array(
 *           'href' => TRUE,
 *           'rel' => array('nofollow' => TRUE),
 *         ),
 *         // Only allows the 'src' and 'alt' attributes on the <alt> tag,
 *         // with any value.
 *         'img' => array(
 *           'src' => TRUE,
 *           'alt' => TRUE,
 *         ),
 *         // Allow RDFa on <span> tags, using only the dc, foaf, xsd and sioc
 *         // vocabularies/namespaces.
 *         'span' => array(
 *           'property' => array('dc:*' => TRUE, 'foaf:*' => TRUE),
 *           'datatype' => array('xsd:*' => TRUE),
 *           'rel' => array('sioc:*' => TRUE),
 *         ),
 *         // Forbid the 'style' and 'on*' ('onClick' etc.) attributes on any
 *         // tag.
 *         '*' => array(
 *           'style' => FALSE,
 *           'on*' => FALSE,
 *         ),
 *       )
 *     )
 *     @endcode
 *
 *   A simpler example, for a very coarse filter:
 *     @code
 *     array(
 *       'forbidden_tags' => array('iframe', 'script')
 *     )
 *     @endcode
 *
 *   The simplest example possible: a filter that doesn't allow any HTML:
 *     @code
 *     array(
 *       'allowed' => array()
 *     )
 *     @endcode
 *
 *   And for a filter that applies no restrictions, i.e. allows any HTML:
 *     @code
 *     FALSE
 *     @endcode
 */
function editor_filter_html_get_html_restrictions($filter) {
  // Parse the allowed HTML setting, and gradually make the list of allowed
  // tags more specific.
  $restrictions = array('allowed' => array());

  // Make all the tags self-closing, so they will be parsed into direct
  // children of the body tag in the DomDocument.
  $html = str_replace('>', ' />', $filter->settings['allowed_html']);
  // Protect any trailing * characters in attribute names, since DomDocument
  // strips them as invalid.
  $star_protector = '__zqh6vxfbk3cg__';
  $html = str_replace('*', $star_protector, $html);
  $body_child_nodes = filter_dom_load($html)->getElementsByTagName('body')->item(0)->childNodes;

  foreach ($body_child_nodes as $node) {
    if ($node->nodeType !== XML_ELEMENT_NODE) {
      // Skip the empty text nodes inside tags.
      continue;
    }
    $tag = $node->tagName;
    if ($node->hasAttributes()) {
      // Mark the tag as allowed, assigning TRUE for each attribute name if
      // all values are allowed, or an array of specific allowed values.
      $restrictions['allowed'][$tag] = array();
      // Iterate over any attributes, and mark them as allowed.
      foreach ($node->attributes as $name => $attribute) {
        // Put back any trailing * on wildcard attribute name.
        $name = str_replace($star_protector, '*', $name);

        // Put back any trailing * on wildcard attribute value and parse out
        // the allowed attribute values.
        $allowed_attribute_values = preg_split('/\s+/', str_replace($star_protector, '*', $attribute->value), -1, PREG_SPLIT_NO_EMPTY);

        // Sanitize the attribute value: it lists the allowed attribute values
        // but one allowed attribute value that some may be tempted to use
        // is specifically nonsensical: the asterisk. A prefix is required for
        // allowed attribute values with a wildcard. A wildcard by itself
        // would mean allowing all possible attribute values. But in that
        // case, one would not specify an attribute value at all.
        $allowed_attribute_values = array_filter($allowed_attribute_values, function ($value) use ($star_protector) {
          return $value !== '*';
        });

        if (empty($allowed_attribute_values)) {
          // If the value is the empty string all values are allowed.
          $restrictions['allowed'][$tag][$name] = TRUE;
        }
        else {
          // A non-empty attribute value is assigned, mark each of the
          // specified attribute values as allowed.
          foreach ($allowed_attribute_values as $value) {
            $restrictions['allowed'][$tag][$name][$value] = TRUE;
          }
        }
      }
    }
    else {
      // Mark the tag as allowed, but with no attributes allowed.
      $restrictions['allowed'][$tag] = FALSE;
    }
  }

  // The 'style' and 'on*' ('onClick' etc.) attributes are always forbidden,
  // and are removed by Xss::filter().
  // The 'lang', and 'dir' attributes apply to all elements and are always
  // allowed. The list of allowed values for the 'dir' attribute is enforced
  // by self::filterAttributes(). Note that those two attributes are in the
  // short list of globally usable attributes in HTML5. They are always
  // allowed since the correct values of lang and dir may only be known to
  // the content author. Of the other global attributes, they are not usually
  // added by hand to content, and especially the class attribute can have
  // undesired visual effects by allowing content authors to apply any
  // available style, so specific values should be explicitly allowed.
  // @see http://www.w3.org/TR/html5/dom.html#global-attributes
  $restrictions['allowed']['*'] = array(
    'style' => FALSE,
    'on*' => FALSE,
    'lang' => TRUE,
    'dir' => array('ltr' => TRUE, 'rtl' => TRUE),
  );

  return $restrictions;
}

/**
 * Implements callback_filter_settings().
 *
 * Filter settings callback for the HTML content filter.
 */
function _editor_html_settings($form, $form_state, $filter, $format, $defaults = array(), $filters = array()) {
  $filter->settings += $defaults;

  $settings['allowed_html'] = array(
    '#type' => 'textfield',
    '#title' => t('Allowed HTML tags'),
    '#default_value' => $filter->settings['allowed_html'],
    '#maxlength' => 2048,
    // Add a class which allows the textfield to be targeted by JavaScript. The
    // element's default ID will change every time it is updated via AJAX, which
    // makes it unsuitable for this purpose.
    '#attributes' => array('class' => array('allowed-html')),
    '#description' => t('A list of HTML tags that can be used. By default only the <em>lang</em> and <em>dir</em> attributes are allowed for all HTML tags. Each HTML tag may have attributes which are treated as allowed attribute names for that HTML tag. Each attribute may allow all values, or only allow specific values. Attribute names or values may be written as a prefix and wildcard like <em>jump-*</em>. JavaScript event attributes, JavaScript URLs, and CSS are always stripped.'),
    '#attached' => array(
      'library' => array(
        array('filter', 'filter.filter_html.admin'),
      ),
    ),
  );
  $settings['filter_html_help'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display basic HTML help in long filter tips'),
    '#default_value' => $filter->settings['filter_html_help'],
  );
  $settings['filter_html_nofollow'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add rel="nofollow" to all links'),
    '#default_value' => $filter->settings['filter_html_nofollow'],
  );

  return $settings;
}
