<?php

/**
 * @file
 * Editor hooks implemented by the Editor CKEditor module.
 */

/**
 * Implements hook_editor_info().
 */
function editor_ckeditor_editor_info() {
  $editors['ckeditor'] = array(
    'label' => t('CKEditor'),
    'supports_content_filtering' => TRUE,
    'supports_inline_editing' => TRUE,
    'is_xss_safe' => FALSE,
    'supported_element_types' => array(
      "textarea"
    ),
    'default settings' => array(
      'toolbar' => array(
        array(
          array(
            'name' => t('Formatting'),
            'items' => array('Bold', 'Italic', 'Blockquote', 'Format'),
          ),
          array(
            'name' => t('Lists'),
            'items' => array('BulletedList', 'NumberedList'),
          ),
          array(
            'name' => t('Alignment'),
            'items' => array('JustifyLeft', 'JustifyCenter', 'JustifyRight'),
          ),
          array(
            'name' => t('Linking'),
            'items' => array('DrupalLink', 'DrupalUnlink'),
          ),
          array(
            'name' => t('Media'),
            'items' => array('DrupalImage'),
          ),
          array(
            'name' => t('Tools'),
            'items' => array('Source', 'Maximize'),
          ),
        ),
      ),
      'plugins' => array(),
    ),
    'file' => 'includes/editor_ckeditor.admin.inc',
    'libraries callback' => 'editor_ckeditor_get_libraries',
    'validate callback' => 'editor_ckeditor_settings_form_validate',
    'submit callback' => 'editor_ckeditor_settings_form_submit',
    'settings callback' => 'editor_ckeditor_settings_form',
    'js settings callback' => 'editor_ckeditor_get_js_settings',
  );

  return $editors;
}

/**
 * Editor JS settings callback; Add CKEditor settings to the page for a format.
 *
 * @param object $format
 *   The filter format object for which CKEditor is adding its settings.
 *
 * @return array
 *   An associative array of CKEditor settings. For a complete list of settings
 *   see http://docs.ckeditor.com/#!/api/CKEDITOR.config-cfg-skin.
 */
function editor_ckeditor_get_js_settings($format) {
  global $language;

  $settings = array();

  $plugins = editor_ckeditor_plugins();

  // Get the settings for all enabled plugins, even the internal ones.
  $enabled_plugins = array_keys(editor_ckeditor_get_enabled_plugin_files($format, TRUE));
  foreach ($enabled_plugins as $plugin_id) {
    $plugin = $plugins[$plugin_id];
    if (!empty($plugin['config callback'])) {
      $settings += $plugin['config callback']($format);
    }
  }

  // Fall back on English if no matching language code was found.
  $display_langcode = 'en';

  // Map the interface language code to a CKEditor translation if interface
  // translation is enabled.
  if (module_exists('locale')) {
    $ckeditor_langcodes = editor_ckeditor_get_langcodes();
    $language_interface = $language->language;

    if (isset($ckeditor_langcodes[$language_interface])) {
      $display_langcode = $ckeditor_langcodes[$language_interface];
    }
  }

  // Next, set the most fundamental CKEditor settings.
  $external_plugin_files = editor_ckeditor_get_enabled_plugin_files($format);

  $settings += array(
    'toolbar' => editor_ckeditor_build_toolbar_js_setting($format),
    'contentsCss' => editor_ckeditor_build_contents_css_js_setting($format),
    'extraPlugins' => implode(',', array_keys($external_plugin_files)),
    'language' => $display_langcode,
    // Configure CKEditor to not load styles.js. The StylesCombo plugin will
    // set stylesSet according to the user's settings, if the "Styles" button
    // is enabled. We cannot get rid of this until CKEditor will stop loading
    // styles.js by default.
    // See http://dev.ckeditor.com/ticket/9992#comment:9.
    'stylesSet' => FALSE,
  );

  // Finally, set Drupal-specific CKEditor settings.
  $root_relative_file_url = function ($uri) {
    return editor_file_url_transform_relative(file_create_url($uri));
  };

  $settings += array(
    'drupalExternalPlugins' => array_map($root_relative_file_url, $external_plugin_files),
  );

  drupal_alter('editor_ckeditor_settings', $settings, $format);

  ksort($settings);

  return $settings;
}

/**
 * Returns a list of language codes supported by CKEditor.
 *
 * @return array
 *   An associative array keyed by language codes.
 */
function editor_ckeditor_get_langcodes() {
  // Cache the file system based language list calculation because this would
  // be expensive to calculate all the time. The cache is cleared on core
  // upgrades which is the only situation the CKEditor file listing should
  // change.
  $langcode_cache = cache_get('editor_ckeditor:langcodes');
  if (!empty($langcode_cache)) {
    $langcodes = $langcode_cache->data;
  }
  if (empty($langcodes)) {
    $langcodes = [];
    // Collect languages included with CKEditor based on file listing.
    $files = scandir(drupal_get_path('module', 'editor_ckeditor') . '/lib/ckeditor/lang');
    foreach ($files as $file) {
      if ($file[0] !== '.' && preg_match('/\.js$/', $file)) {
        $langcode = basename($file, '.js');
        $langcodes[$langcode] = $langcode;
      }
    }
    cache_set('editor_ckeditor:langcodes', $langcodes);
  }

  // Get language mapping if available to map to Drupal language codes.
  // This is configurable via an alter hook and not expensive to get, so
  // we don't include it in the cached language list.
  $language_mappings = module_exists('locale') ? editor_get_browser_drupal_langcode_mappings() : array();

  foreach ($langcodes as $langcode) {
    // If this language code is available in a Drupal mapping, use that to
    // compute a possibility for matching from the Drupal langcode to the
    // CKEditor langcode.
    // For instance, CKEditor uses the langcode 'no' for Norwegian, Drupal
    // uses 'nb'. This would then remove the 'no' => 'no' mapping and replace
    // it with 'nb' => 'no'. Now Drupal knows which CKEditor translation to
    // load.
    if (isset($language_mappings[$langcode]) && !isset($langcodes[$language_mappings[$langcode]])) {
      $langcodes[$language_mappings[$langcode]] = $langcode;
      unset($langcodes[$langcode]);
    }
  }

  return $langcodes;
}

/**
 * Returns a list of libraries this plugin requires.
 *
 * These libraries will be attached to the text_format element on which the
 * editor is being loaded.
 *
 * @param $format
 *   An object representing a text format.
 *
 * @return array
 *   An array of libraries suitable for usage in a render API #attached
 *   property.
 */
function editor_ckeditor_get_libraries($format) {
  $libraries = array(
    array('editor_ckeditor', 'editor_ckeditor'),
  );

  $plugins = editor_ckeditor_plugins();

  // Get the required libraries for any enabled plugins.
  $enabled_plugins = array_keys(editor_ckeditor_get_enabled_plugin_files($format));

  foreach ($enabled_plugins as $plugin_id) {
    $plugin = $plugins[$plugin_id];
    if (!empty($plugin['libraries'])) {
      $additional_libraries = array_udiff($plugin['libraries'], $libraries, function($a, $b) {
        return $a[0] === $b[0] && $a[1] === $b[1] ? 0 : 1;
      });
      $libraries = array_merge($libraries, $additional_libraries);
    }
  }

  return $libraries;
}

/**
 * Builds the "toolbar" configuration part of the CKEditor JS settings.
 *
 * @see editor_ckeditor_build_contents_css_js_setting()
 *
 * @param $format
 *   An object representing a text format.
 *
 * @return array
 *   An array containing the "toolbar" configuration.
 */
function editor_ckeditor_build_toolbar_js_setting($format) {
  $toolbar = array();

  $settings = $format->editor_settings;
  foreach ($settings['toolbar'] as $row) {
    foreach ($row as $group) {
      $toolbar[] = $group;
    }
    $toolbar[] = '/';
  }

  // Remove the trailing slash (end row) from the toolbar.
  if ($toolbar) {
    array_pop($toolbar);
  }

  return $toolbar;
}

/**
 * Builds the "contentsCss" configuration part of the CKEditor JS settings.
 *
 * @param $format
 *   An object representing a text format.
 *
 * @return array
 *   An array containing the "contentsCss" configuration.
 */
function editor_ckeditor_build_contents_css_js_setting($format) {
  $css = array(
    drupal_get_path('module', 'editor_ckeditor') . '/css/ckeditor-iframe.css',
    drupal_get_path('module', 'editor') . '/css/components/align.module.css',
    drupal_get_path('module', 'editor') . '/css/components/underline.module.css',
  );

  drupal_alter('editor_ckeditor_css', $css, $format);

  // Get a list of all enabled plugins' iframe instance CSS files.
  $plugins_css = array_reduce(editor_ckeditor_get_css_files($format), function ($result, $item) {
    return array_merge($result, array_values($item));
  }, array());
  $css = array_merge($css, $plugins_css);
  $css = array_merge($css, _editor_ckeditor_theme_css());
  $query_string = variable_get('css_js_query_string', '0');
  $css = array_map(function ($item) use ($query_string) {
    $query_string_separator = (strpos($item, '?') !== FALSE) ? '&' : '?';
    return $item . $query_string_separator . $query_string;
  }, $css);

  $css = array_map('editor_file_create_url', $css);
  $css = array_map('editor_file_url_transform_relative', $css);

  return array_values($css);
}

/**
 * Injects the CKEditor plugins settings forms as a vertical tabs subform.
 *
 * @param $form
 *   A reference to an associative array containing the structure of the form.
 * @param $form_state
 *   The current state of the form.
 * @param object $format
 *   A text format object.
 */
function editor_ckeditor_inject_plugin_settings_form(&$form, &$form_state, $format) {
  $plugins = editor_ckeditor_plugins();

  foreach (array_keys($plugins) as $plugin_id) {
    $plugin = $plugins[$plugin_id];

    if (!empty($plugin['supports_configuration'])) {
      $plugin_settings_form = array();

      $form['plugins'][$plugin_id] = array(
        '#type' => 'fieldset',
        '#title' => $plugin['label'],
        '#open' => TRUE,
        '#group' => 'editor][settings][plugin_settings',
        '#attributes' => array(
          'data-ckeditor5-plugin-id' => $plugin_id,
        ),
      );
      // Provide enough metadata for the drupal.ckeditor.admin library to
      // allow it to automatically show/hide the vertical tab containing the
      // settings for this plugin. Only do this if it's a CKEditor plugin that
      // just provides buttons, don't do this if it's a contextually enabled
      // CKEditor plugin. After all, in the latter case, we can't know when
      // its settings should be shown!
      if (!empty($plugin['buttons']) && empty($plugin['contextual'])) {
        $form['plugins'][$plugin_id]['#attributes']['data-ckeditor-buttons'] = implode(' ', array_keys($plugin['buttons']));
        $form['plugins'][$plugin_id]['#attributes']['data-ckeditor-feature-dependency'] = implode(' ', array_keys($plugin['buttons']));
      }

      $form['plugins'][$plugin_id] += $plugin['settings form']($plugin_settings_form, $form_state, $format);
    }
  }
}

/**
 * Retrieves enabled plugins' iframe instance CSS files, keyed by plugin ID.
 *
 * @param $format
 *   An object representing a text format.
 *
 * @return array()
 *   Enabled plugins CKEditor CSS files, with plugin IDs as keys and CSS file
 *   paths relative to the Drupal root (as implemented by
 *   editor_ckeditor_get_css_files()) as values.
 */
function editor_ckeditor_get_css_files($format) {
  $plugins = editor_ckeditor_plugins();
  $enabled_plugins = array_keys(editor_ckeditor_get_enabled_plugin_files($format, TRUE));
  $css_files = array();

  foreach ($enabled_plugins as $plugin_id) {
    $plugin = $plugins[$plugin_id];
    if (!empty($plugin['css'])) {
      $css_files[$plugin_id] = $plugin['css'];
    }
  }

  return $css_files;
}
